#include "ADXL362.h"
#include <SPI.h>
#include "SerialLedController.h"
#include <math.h>

/* ボード情報 ****************************************************/

/*  ポート使用状況
  [ 0] Serial RX (Console / LED Controller)
  [ 1] Serial TX (Console / LED Controller)
  [ 2] 
  [ 3] 
  [ 4] LED Controller Reset
  [ 5] PWM Out
  [ 6] LED Controller Baudrate
  [ 7] XBee TX
  [ 8] XBee RX
  [ 9] Mode Switch
  [10] SS (ADXL362)
  [11] MOSI (ADXL362)
  [12] MISO (ADXL362)
  [13] CK (ADXL362)
  [A0] Analog Read
  [A1] 
  [A2] 
  [A3] 
  [A4] 
  [A5] 
  [A6] 
  [A7] 
*/

/* GR-KURUMI シリアルポート [TX] [RX]
  Serial  [ 1] [ 0]
  Serial1 [ 7] [ 8]
  Serial2 [ 9] [10] SPI同時不可
*/

/* 定数 ****************************************************/
 
//#define DEBUG           // 有効にすると各種情報をSerialに流し、ステータスはオンボードのLEDで表示。LED演出は止まる。

#define LED_ON false
#define LED_OFF true
#define XBEE Serial1      // XBee用シリアル
#define CONSOLE Serial    // デバッグ用シリアル
#define LEDCONTROL Serial // LED用シリアル
#define INPORT A0         // 入力ポート
#define OUTPORT 5         // 出力ポート
#define ANALOG_WRITE_FREQUENCY 8000 
                          //analogWriteの周波数
#define LPFK 0.3          // LPFの係数
#define INT_STATE_MILLI 100 // 状態を確認する間隔
#define INT_SAMPLING_MICRO 250 // サンプリングレート
#define Mode_Switch 9     // 手動スイッチポート
#define REG_RECIVE 20     // 受信モードの角度
#define LED_RED     22    // 基板上のフルカラーLEDポート
#define LED_GREEN   23
#define LED_BLUE    24
#define MODE_SLEEP 0      // スリープモード
#define MODE_TRANSLATE 1  // 送信モード
#define MODE_RECIVE 2     // 受信モード


/* LED演出関係 ****************************************************/

// http://carcon999.web.fc2.com/ledprogrammer/index.html
const uint8_t led_play_list_Left[][SERIALCOMMAND_FIXEDLEN] = 
{ 
  { 0x94, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /* {"name":"script","ctrl":148,"mode":1} */
  { 0x90, 0x01, 0x00, 0x00, 0x1e, 0x00, 0x00, 0x00 }, /* {"name":"system","ctrl":144,"ledtype":1,"length":30,"wait":0} */
  { 0x84, 0x0d, 0x00, 0x0d, 0x01, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":true,"off":true,"h":300,"s":255,"v":13,"target":1} */
  { 0x80, 0x09, 0x00, 0x09, 0x02, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":true,"off":false,"h":300,"s":255,"v":9,"target":2} */
  { 0x80, 0x06, 0x00, 0x06, 0x04, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":true,"off":false,"h":300,"s":255,"v":6,"target":4} */
  { 0x88, 0x03, 0x00, 0x03, 0x08, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":false,"off":false,"h":300,"s":255,"v":3,"target":8} */
  { 0x92, 0x01, 0x01, 0x00, 0xff, 0xff, 0x1e, 0x00 }, /* {"name":"shift","ctrl":146,"dir":1,"mode":1,"count":65535,"wait":30} */
  { 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /* {"name":"script","ctrl":148,"mode":0} */
};

const uint8_t led_play_list_Right[][SERIALCOMMAND_FIXEDLEN] = 
{ 
  { 0x94, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /* {"name":"script","ctrl":148,"mode":1} */
  { 0x90, 0x01, 0x00, 0x00, 0x1e, 0x00, 0x00, 0x00 }, /* {"name":"system","ctrl":144,"ledtype":1,"length":30,"wait":0} */
  { 0x84, 0x00, 0x0b, 0x0c, 0x08, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":true,"off":true,"h":185,"s":255,"v":12,"target":8} */
  { 0x80, 0x00, 0x09, 0x09, 0x04, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":true,"off":false,"h":180,"s":255,"v":9,"target":4} */
  { 0x80, 0x00, 0x06, 0x06, 0x02, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":true,"off":false,"h":180,"s":255,"v":6,"target":2} */
  { 0x88, 0x00, 0x03, 0x03, 0x01, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":false,"off":false,"h":180,"s":255,"v":3,"target":1} */
  { 0x92, 0x00, 0x01, 0x00, 0xff, 0xff, 0x1e, 0x00 }, /* {"name":"shift","ctrl":146,"dir":0,"mode":1,"count":65535,"wait":30} */
  { 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /* {"name":"script","ctrl":148,"mode":0} */
}; 

const uint8_t led_play_list_BLACK[][SERIALCOMMAND_FIXEDLEN] = 
{ 
  { 0x94, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /* {"name":"script","ctrl":148,"mode":1} */
  { 0x90, 0x01, 0x00, 0x00, 0x1e, 0x00, 0x00, 0x00 }, /* {"name":"system","ctrl":144,"ledtype":1,"length":30,"wait":0} */
  { 0x8c, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00 }, /* {"name":"color","ctrl":128,"tr":false,"off":true,"h":179,"s":0,"v":0,"target":1} */
  { 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /* {"name":"script","ctrl":148,"mode":0} */
}; 

#ifndef DEBUG
// シリアルコントローラ用ライブラリのインスタンスを生成する。 
// リセットとボーレート指定用のPIN番号を指定する。もし利用しない場合は－１を指定する。 
SerialLedController controller(4, 6); 
#endif




/* 割込み用ハンドラ ****************************************************/

// Flg_chk_state は状態（角度）を確認するためのフラグ
// on_check_state(long unsigned int time) はタイマー割り込みで呼び出され、Flg_chk_state を true にするハンドラ関数。

boolean Flg_chk_state = false;
void on_check_state(long unsigned int time){
  // フラグを立てる
  Flg_chk_state = true;
}


// Flg_comm は true の時に送受信を行うフラグ。
// on_comm(void) はタイマー割り込みで呼び出され、Flg_comm を true にするハンドラ関数。

boolean  Flg_comm = false; 
void on_comm(void){
  // フラグを立てる
  Flg_comm = true;
}



/* 加速度センサー関係 ****************************************************/

ADXL362 xl;
int XValue, YValue, ZValue, Temperature; // センサーからのデータ
float XValue_LPF, YValue_LPF, ZValue_LPF; // LPFを通した値　ノイズ除去後

// 加速度センサーをからデータを取得し、本体の角度を得る。
// 角度から状態を判別し、現在のモードを返す。
int read_sensor(void){

  float xy, seeta;

  // 加速度センサーからデータを取得
  xl.readXYZTData(&XValue, &YValue, &ZValue, &Temperature); 

  // LPFをかけてノイズ除去　（効き具合はLPFKの値で調整する）
  XValue_LPF = XValue_LPF * (1-LPFK) + XValue * LPFK ;
  YValue_LPF = YValue_LPF * (1-LPFK) + YValue * LPFK ;
  ZValue_LPF = ZValue_LPF * (1-LPFK) + ZValue * LPFK ;

  xy = (fabs(XValue_LPF)+fabs(YValue_LPF)); // x-y面の加速度
  if(xy){
    seeta = atan(ZValue_LPF/xy)*180/PI; // z-xyの角度
  }else{
    seeta = (ZValue_LPF>0) ? 90 : -90;
  }

#ifdef DEBUG
  CONSOLE.print("X:");
  CONSOLE.print(XValue_LPF);
  CONSOLE.print(" Y:");
  CONSOLE.print(YValue_LPF);
  CONSOLE.print(" Z:");
  CONSOLE.print(ZValue_LPF);
  CONSOLE.print(" xy:");
  CONSOLE.print(xy);
  CONSOLE.print(" DEG:");
  CONSOLE.println(seeta);
#endif

  if( (seeta>-REG_RECIVE) && (seeta<REG_RECIVE) ){
    return MODE_RECIVE;
  }else if( (seeta<-80) ){
    return MODE_SLEEP;
  }else{
    return MODE_TRANSLATE;
  }

}


/* モード判定処理 ****************************************************/

// 現在のモード
int NowMode;

// 角度によってモードを切り替える
// 通常はテープLEDによる演出をするが、DEBUG が有効の時にはオンボードのLEDの色を変える。
void check_state(void){

  // 状況によってモードを切り替える
  switch(read_sensor()){
  
    case MODE_TRANSLATE: // 送信モード
      if(NowMode != MODE_TRANSLATE){
        NowMode = MODE_TRANSLATE; // 送信モードに変更
        mode_translate_init(); // 初期化
        
#ifndef DEBUG
        // LED演出
        controller.stop(); // 演出を止める
        // 演出用のテーブルを与える。（コントローラにデータを転送する）
        if(controller.write(led_play_list_Left, sizeof(led_play_list_Left))){ 
          // 転送に成功したら演出を開始する。 
          controller.start(); 
        }
#else
        // デバッグモードではオンボードのLEDを点灯する
        digitalWrite(LED_RED, LED_ON);
        digitalWrite(LED_GREEN, LED_OFF);
        digitalWrite(LED_BLUE, LED_OFF);
#endif
      }
      break;
      
    case MODE_RECIVE: // 受信モード
      if(NowMode != MODE_RECIVE){
        NowMode = MODE_RECIVE; // 受信モードに変更
        mode_recive_init(); // 初期化
        
#ifndef DEBUG
        // LED演出
        controller.stop(); // 演出を止める
        // 演出用のテーブルを与える。（コントローラにデータを転送する）   
        if(controller.write(led_play_list_Right, sizeof(led_play_list_Right))){ 
          // 転送に成功したら演出を開始する。 
          controller.start(); 
        }
#else
        // デバッグモードでは基板上のLEDを点灯する
        digitalWrite(LED_RED, LED_OFF);
        digitalWrite(LED_GREEN, LED_ON);
        digitalWrite(LED_BLUE, LED_OFF);
#endif
      }
      break;
      
    case MODE_SLEEP: // スリープモード
      if(NowMode != MODE_SLEEP){
        NowMode = MODE_SLEEP; // スリープモードにする
        mode_sleep_init(); // 初期化
        
#ifndef DEBUG
        // LED演出
        controller.stop(); // 演出を止める
        // 演出用のテーブルを与える。（コントローラにデータを転送する）   
        if(controller.write(led_play_list_BLACK, sizeof(led_play_list_BLACK))){ 
          // 転送に成功したら演出を開始する。 
          controller.start(); 
        }
#else
        // デバッグモードでは基板上のLEDを点灯する
        digitalWrite(LED_RED, LED_OFF);
        digitalWrite(LED_GREEN, LED_OFF);
        digitalWrite(LED_BLUE, LED_ON);
#endif
      }
      break;
  }
  
  // フラグを下ろす
  Flg_chk_state = false;
}




/* モード別処理 （スリープ）****************************************************/

// スリープ動作初期化
void mode_sleep_init(void){
}

// スリープ動作
void mode_sleep(void){
  if( Flg_comm ){
    
    Flg_comm = false;
  }
}


/* モード別処理 （送信）****************************************************/

// 送信動作初期化
void mode_translate_init(void){

  pinMode(OUTPORT, INPUT);

  // バッファクリア
  XBEE.flush();
}

// 送信動作
void mode_translate(void){
  if( Flg_comm ){
    XBEE.write((unsigned char)analogRead(INPORT));
    
    Flg_comm = false;
  }
}


/* モード別処理 （受信）****************************************************/

// 受信動作初期化
void mode_recive_init(void){

  pinMode(OUTPORT, OUTPUT);
  
  // バッファクリア
  XBEE.flush();
}

// 受信動作
void mode_recive(void){

  if( Flg_comm ){
  
    if(XBEE.available()){
        unsigned char c = XBEE.read();
        analogWrite(OUTPORT,c);
    }
    
    Flg_comm = false;
  }

}


/* 初期設定 ****************************************************/

void setup() {

  // 加速度センサー初期化
  xl.begin();                   // Setup SPI protocol, issue device soft reset
  xl.beginMeasure();            // Switch ADXL362 to measure mode  
  xl.checkAllControlRegs();     // Burst Read all Control Registers, to check for proper setup

#ifdef DEBUG
  // コンソール
  CONSOLE.begin(115200);
#else
  // LEDコントローラ
  // シリアルクラスと、通信速度（true:57600, false:19200）を指定する 
  controller.begin(&LEDCONTROL, true); 
#endif

  // XBee初期化
  XBEE.begin(115200);
  
  // PWM周波数設定
  analogWriteFrequency(ANALOG_WRITE_FREQUENCY);
  
  // 割込み設定
  attachCyclicHandler(0,on_check_state,INT_STATE_MILLI);
  attachMicroIntervalTimerHandler(on_comm, INT_SAMPLING_MICRO);
  
  // デバッグ用ポート
  pinMode(Mode_Switch,INPUT_PULLUP);
  
  // デバッグ用LED
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);
  digitalWrite(LED_RED, LED_OFF);
  digitalWrite(LED_GREEN, LED_OFF);
  digitalWrite(LED_BLUE, LED_OFF);
       
}


/* メインループ ****************************************************/

void loop() {

  if(Flg_chk_state)check_state();
  
  switch( NowMode ){
    case MODE_SLEEP :
      mode_sleep();
      break;
    case MODE_TRANSLATE :
      mode_translate();
      break;
    case MODE_RECIVE :
      mode_recive();
      break;
  }

}
